==> gkligo-0.0.8.tar.gz: gkligo-0.0.8/gkligo/scripts/python/downloadGWAlerts.py <==
#!/usr/bin/env python
"""
Download GW Alerts (multi-order skymap) and convert to MOC file if required.
https://emfollow.docs.ligo.org/userguide/tutorial/multiorder_skymaps.html

This code written by Ken Smith based on code by Leo Singer and Roy Williams 

Requires:
numpy
ligo.skymap
astropy
mocpy

Usage:
  %s <configFile> <action> [--writeMap] [--writeMOC] [--directory=<directory>] [--contours=<contours>] [--pidfile=<pidfile>] [--logfile=<logfile>] [--daemonErrFile=<deamonErrFile>] [--daemonOutFile=<deamonOutFile>] [--terminal]
  %s (-h | --help)
  %s --version

where action is start|stop|restart

Options:
  -h --help                         Show this screen.
  --version                         Show version.
  --writeMap                        Write the Map file
  --writeMOC                        Write the MOC file (selected contour)
  --directory=<directory>           Directory to where the maps and MOCs will be written [default: /tmp].
  --contours=<contours>             Which MOC contours do you want? Multiple contours should be separated by commas, with no spaces [default: 90]
  --pidfile=<pidfile>               PID file [default: /tmp/ligo.pid]
  --logfile=<logfile>               PID file [default: /tmp/ligo.log]
  --daemonErrFile=<daemonErrFile>   Daemon Error File - for recording unexpected errors [default: /tmp/ligodaemonerr.log].
  --daemonOutFile=<daemonOutFile>   Daemon Out File - for recording unexpected output [default: /tmp/ligodaemonout.log].
  --terminal                        Override the Daemon error and out files and write to the terminal. 

E.g.:
  %s config.yaml start --directory=/home/atls/ligo --writeMap
  %s config.yaml start --writeMap --writeMOC --directory=/tmp --contours=90
  %s config.yaml start --writeMOC --directory=/tmp --contours=90,50,10

"""
import sys
__doc__ = __doc__ % (sys.argv[0], sys.argv[0], sys.argv[0], sys.argv[0], sys.argv[0], sys.argv[0])
from docopt import docopt

from gcn_kafka import Consumer
import json
import base64
from gkutils.commonutils import Struct, cleanOptions
from io import BytesIO
import daemon
from daemon import pidfile
import signal
import os
import time
import logging

def shutdown(signum, frame):  # signum and frame are mandatory
    """shutdown.

    Args:
        signum:
        frame:
    """
    sys.stdout.write("\nOuch...\n")
    sys.exit(0)


def writeMOC(inputFilePointer, outputMOCName, contour, logger):
    """writeMOC.

    Args:
        inputFilePointer:
        outputMOCName:
        contour:
        logger:
    """
    from astropy.table import Table
    from astropy import units as u
    import astropy_healpix as ah
    import numpy as np
    import math
    from ligo.skymap.moc import uniq2pixarea

    # Read and verify the input
    skymap = Table.read(inputFilePointer, format='fits')
    #print('Input multi-order skymap:')
    logger.info(skymap.info)

    # Sort by prob density of pixel
    skymap.sort('PROBDENSITY', reverse=True)

    # Get area*probdensity for each pixel
    pixel_area = uniq2pixarea(skymap['UNIQ'])
    #print('Total area = %.1f\n' % (np.sum(pixel_area) * (180/math.pi)**2))

    # Probability per pixel
    prob = pixel_area*skymap['PROBDENSITY']
    cumprob = np.cumsum(prob)

    # Should be 1.0. But need not be.
    sumprob = np.sum(prob)
    logger.info('Sum probability = %.3f\n' % sumprob)

    # Find the index where contour of prob is inside
    i = cumprob.searchsorted(contour*sumprob)
    area_wanted = pixel_area[:i].sum()
    logger.info('Area of %.2f contour is %.2f sq deg' % \
        (contour, area_wanted * (180/math.pi)**2))

    # A MOC is just an astropy Table with one column of healpix indexes
    skymap = skymap[:i]
    skymap = skymap['UNIQ',]
    logger.info(skymap.info)
    skymap.write(outputMOCName, format='fits', overwrite=True)
    logger.info('MOC file %s written' % outputMOCName)


def listen(options):
    """listen.

    Args:
        options:
    """
    pid = os.getpid()
    sys.stdout.write("\nListening... PID is %s\n" % pid)
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.INFO)

    fh = logging.FileHandler(options.logfile)
    fh.setLevel(logging.INFO)

    formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    formatter = logging.Formatter(formatstr)

    fh.setFormatter(formatter)

    logger.addHandler(fh)

    import yaml
    with open(options.configFile) as yaml_file:
        config = yaml.safe_load(yaml_file)

    client_id = config['client_id']
    client_secret = config['client_secret']
    topic = config['topics']

    consumer = Consumer(client_id=client_id,
                        client_secret=client_secret)

    consumer.subscribe(['igwn.gwalert'])

    while True:
        # Non zero timeout is required, otherwise consume blocks termination signals.
        for message in consumer.consume(timeout=5):
            dataDict = json.loads(message.value().decode('utf-8'))
            try:
                superEventId = dataDict['superevent_id']
                alertTimeStamp = dataDict['time_created'].replace(' ','T')
                alertType = dataDict['alert_type']
                alertName = superEventId + '_' + alertType + '_' + alertTimeStamp
                logger.info("Alert Received: %s" % alertName) # Future version of this script will log the output.
                if dataDict['event'] is not None and options.writeMap:
                    skymap = dataDict['event']['skymap']
                    with open(options.directory + '/' + alertName + '.fits', 'wb') as fitsFile:
                        fitsFile.write(base64.b64decode(skymap))

                if dataDict['event'] is not None and options.writeMOC:
                    for contour in options.contours.split(','):
                        skymap = dataDict['event']['skymap']
                        try:
                            c = float(contour)/100.0
                            writeMOC(BytesIO(base64.b64decode(skymap)), options.directory + '/' + alertName + '_' + contour + '.moc', c, logger)
                        except ValueError as e:
                            logger.error("Contour %s is not a float" % contour)


            except KeyError as e:
                logger.error(e)
                pass
            logger.info("")


def startDaemon(options):
    """startDaemon.

    Args:
        options:
    """

    if options.terminal:
        err = sys.stderr
        out = sys.stdout
    else:
        out = open(options.daemonOutFile, 'w')
        err = open(options.daemonErrFile, 'w')

    with daemon.DaemonContext(
        working_directory='/tmp',
        umask=0o002,
        pidfile=pidfile.TimeoutPIDLockFile(options.pidfile),
        stdout=out,
        stderr=err,
        signal_map={ signal.SIGTERM: shutdown }
        ) as context:
        listen(options)


def main():
    """main.
    """
    opts = docopt(__doc__, version='0.1')
    opts = cleanOptions(opts)

    # Use utils.Struct to convert the dict into an object for compatibility with old optparse code.
    options = Struct(**opts)


    if options.action not in ['start', 'stop', 'restart']:
        sys.stderr.write('Valid options for action are start|stop|restart\n')
        sys.exit(1)

    if options.action == 'start':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
                sys.stderr.write("Daemon is already running (PID = %s). Stop and restart if you want to restart it.\n" % pid)
        else:
            startDaemon(options)

    if options.action == 'stop':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
            print("Stopping daemon (PID = %s)." % pid)
            os.kill(int(pid), signal.SIGTERM)
        else:
            sys.stderr.write("Daemon is not running.\n")


    if options.action == 'restart':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
            print("Stopping daemon (PID = %s)." % pid)
            os.kill(int(pid), signal.SIGTERM)
            time.sleep(5)
            startDaemon(options)
        else:
            startDaemon(options)



if __name__ == '__main__':
    main()

==> gkligo-0.0.9.tar.gz: gkligo-0.0.9/gkligo/scripts/python/downloadGWAlerts.py <==
#!/usr/bin/env python
"""
Download GW Alerts (multi-order skymap) and convert to MOC file if required.
https://emfollow.docs.ligo.org/userguide/tutorial/multiorder_skymaps.html

This code written by Ken Smith based on code by Leo Singer and Roy Williams 

Requires:
numpy
ligo.skymap
astropy
mocpy

Usage:
  %s <configFile> <action> [--writeMap] [--writeMOC] [--directory=<directory>] [--contours=<contours>] [--pidfile=<pidfile>] [--logfile=<logfile>] [--daemonErrFile=<deamonErrFile>] [--daemonOutFile=<deamonOutFile>] [--terminal]
  %s (-h | --help)
  %s --version

where action is start|stop|restart|listen

If action is listen, start in non-daemon mode. Otherwise use daemon mode.

Options:
  -h --help                         Show this screen.
  --version                         Show version.
  --writeMap                        Write the Map file
  --writeMOC                        Write the MOC file (selected contour)
  --directory=<directory>           Directory to where the maps and MOCs will be written [default: /tmp].
  --contours=<contours>             Which MOC contours do you want? Multiple contours should be separated by commas, with no spaces [default: 90]
  --pidfile=<pidfile>               PID file [default: /tmp/ligo.pid]
  --logfile=<logfile>               PID file [default: /tmp/ligo.log]
  --daemonErrFile=<daemonErrFile>   Daemon Error File - for recording unexpected errors [default: /tmp/ligodaemonerr.log].
  --daemonOutFile=<daemonOutFile>   Daemon Out File - for recording unexpected output [default: /tmp/ligodaemonout.log].
  --terminal                        Override the Daemon error and out files and write to the terminal. 

E.g.:
  %s config.yaml start --directory=/home/atls/ligo --writeMap
  %s config.yaml start --writeMap --writeMOC --directory=/tmp --contours=90
  %s config.yaml start --writeMOC --directory=/tmp --contours=90,50,10

"""
import sys
__doc__ = __doc__ % (sys.argv[0], sys.argv[0], sys.argv[0], sys.argv[0], sys.argv[0], sys.argv[0])
from docopt import docopt
from __version__ import __version__

from gcn_kafka import Consumer
import json
import base64
from gkutils.commonutils import Struct, cleanOptions
from io import BytesIO
import daemon
from daemon import pidfile
import signal
import os
import time
import logging

def shutdown(signum, frame):  # signum and frame are mandatory
    """shutdown.

    Args:
        signum:
        frame:
    """
    sys.stdout.write("\nOuch...\n")
    sys.exit(0)


def writeMOC(inputFilePointer, outputMOCName, contour, logger):
    """writeMOC.

    Args:
        inputFilePointer:
        outputMOCName:
        contour:
        logger:
    """
    from astropy.table import Table
    from astropy import units as u
    import astropy_healpix as ah
    import numpy as np
    import math
    from ligo.skymap.moc import uniq2pixarea

    # Read and verify the input
    skymap = Table.read(inputFilePointer, format='fits')
    #print('Input multi-order skymap:')
    logger.info(skymap.info)

    # Sort by prob density of pixel
    skymap.sort('PROBDENSITY', reverse=True)

    # Get area*probdensity for each pixel
    pixel_area = uniq2pixarea(skymap['UNIQ'])
    #print('Total area = %.1f\n' % (np.sum(pixel_area) * (180/math.pi)**2))

    # Probability per pixel
    prob = pixel_area*skymap['PROBDENSITY']
    cumprob = np.cumsum(prob)

    # Should be 1.0. But need not be.
    sumprob = np.sum(prob)
    logger.info('Sum probability = %.3f\n' % sumprob)

    # Find the index where contour of prob is inside
    i = cumprob.searchsorted(contour*sumprob)
    area_wanted = pixel_area[:i].sum()
    logger.info('Area of %.2f contour is %.2f sq deg' % \
        (contour, area_wanted * (180/math.pi)**2))

    # A MOC is just an astropy Table with one column of healpix indexes
    skymap = skymap[:i]
    skymap = skymap['UNIQ',]
    logger.info(skymap.info)
    skymap.write(outputMOCName, format='fits', overwrite=True)
    logger.info('MOC file %s written' % outputMOCName)


def listen(options):
    """listen.

    Args:
        options:
    """
    pid = os.getpid()
    sys.stdout.write("\nListening... PID is %s\n" % pid)
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.INFO)

    fh = logging.FileHandler(options.logfile)
    fh.setLevel(logging.INFO)

    formatstr = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    formatter = logging.Formatter(formatstr)

    fh.setFormatter(formatter)

    logger.addHandler(fh)

    import yaml
    with open(options.configFile) as yaml_file:
        config = yaml.safe_load(yaml_file)

    client_id = config['client_id']
    client_secret = config['client_secret']
    topic = config['topics']

    consumer = Consumer(client_id=client_id,
                        client_secret=client_secret)

    consumer.subscribe(['igwn.gwalert'])

    while True:
        # Non zero timeout is required, otherwise consume blocks termination signals.
        for message in consumer.consume(timeout=5):
            dataDict = json.loads(message.value().decode('utf-8'))
            try:
                superEventId = dataDict['superevent_id']
                alertTimeStamp = dataDict['time_created'].replace(' ','T')
                alertType = dataDict['alert_type']
                alertName = superEventId + '_' + alertType + '_' + alertTimeStamp
                logger.info("Alert Received: %s" % alertName) # Future version of this script will log the output.
                if dataDict['event'] is not None and options.writeMap:
                    skymap = dataDict['event']['skymap']
                    with open(options.directory + '/' + alertName + '.fits', 'wb') as fitsFile:
                        fitsFile.write(base64.b64decode(skymap))

                if dataDict['event'] is not None and options.writeMOC:
                    for contour in options.contours.split(','):
                        skymap = dataDict['event']['skymap']
                        try:
                            c = float(contour)/100.0
                            writeMOC(BytesIO(base64.b64decode(skymap)), options.directory + '/' + alertName + '_' + contour + '.moc', c, logger)
                        except ValueError as e:
                            logger.error("Contour %s is not a float" % contour)


            except KeyError as e:
                logger.error(e)
                pass
            logger.info("")


def startDaemon(options):
    """startDaemon.

    Args:
        options:
    """

    if options.terminal:
        err = sys.stderr
        out = sys.stdout
    else:
        out = open(options.daemonOutFile, 'w')
        err = open(options.daemonErrFile, 'w')

    with daemon.DaemonContext(
        working_directory='/tmp',
        umask=0o002,
        pidfile=pidfile.TimeoutPIDLockFile(options.pidfile),
        stdout=out,
        stderr=err,
        signal_map={ signal.SIGTERM: shutdown }
        ) as context:
        listen(options)


def main():
    """main.
    """
    opts = docopt(__doc__, version=__version__)
    opts = cleanOptions(opts)

    # Use utils.Struct to convert the dict into an object for compatibility with old optparse code.
    options = Struct(**opts)


    if options.action not in ['start', 'stop', 'restart', 'listen']:
        sys.stderr.write('Valid options for action are start|stop|restart|listen\n')
        sys.exit(1)

    # First check that the listen option has been chosen. If so, start listening to the Kafka
    # stream immediately, without daemonisation.

    if options.action == 'listen':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
                sys.stderr.write("\nDaemon is already running (PID = %s). Kill the existing daemon first. E.g. use the stop option.\n" % pid)
        else:
            listen(options)
    
    if options.action == 'start':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
                sys.stderr.write("\nDaemon is already running (PID = %s). Stop and restart if you want to restart it.\n" % pid)
        else:
            startDaemon(options)

    if options.action == 'stop':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
            print("Stopping daemon (PID = %s)." % pid)
            os.kill(int(pid), signal.SIGTERM)
        else:
            sys.stderr.write("\nDaemon is not running.\n")


    if options.action == 'restart':
        if os.path.exists(options.pidfile):
            with open(options.pidfile, mode='r') as f:
                pid = f.read().strip()
            print("Stopping daemon (PID = %s)." % pid)
            os.kill(int(pid), signal.SIGTERM)
            time.sleep(5)
            startDaemon(options)
        else:
            startDaemon(options)



if __name__ == '__main__':
    main()
